/*
 * Copyright (c) 1998, 2000, 2004 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Sugih Jamin (jamin@eecs.umich.edu)
*/
#ifndef lint
static char rcsid[] =
"@(#) $Id: _vcmp.c,v 1.4 2000/10/13 21:18:44 sugih Exp $ (UM)";
#endif

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "vnet.h"
#include "vip.h"
#include "vcmp.h"

/* VCMP error messages, add errcode to vcmp.h when adding new messages.
char *_vcmp_errmsg[] = {
  "Hop count exceeded",
  "Host unreachable",
  "Link down",
  "Unknown protocol",
  "Bad error code",
};
*/
/*
 * vcmp_input(src, vcmp)
 *
 * src points to whence the vcmp pkt came from.
 * vcmp points to the vcmp pkt.
 * Both in network byte-order.
 *
 * Prints out the contents of the vcmp pkt,
 * mapping the vcmp error code to its corresponding
 * error message.  Assumes existence of
 * _vcmp_errmsg global variable.
 * Nothing is modified.
*/
void
_vcmp_input(vin_addr_t *src, vcmp_t *vcmp)
{
  struct hostent *host;
  int errcode;

  if (ntohs(vcmp->vcmp_ver) == VCMP_VERSION) {

    host = gethostbyaddr((char *) &src->vad_ipv4,
                         sizeof(struct in_addr), AF_INET);
    
    errcode = ntohs(vcmp->vcmp_code);
    if (errcode < 0 || errcode > VCMP_ELAST) {
      errcode = VCMP_ELAST;
    }
      
    fprintf(stderr, "VCMP from vrouter %s.%d: %s\nOriginal VIP header:\n", host->h_name, ntohs(src->vad_port), vcmp_errmsg(errcode));
    fprintf(stderr, "ver: %d; hlim: %d\n", ntohs(vcmp->vcmp_vip.vip_ver),
            ntohs(vcmp->vcmp_vip.vip_hop));
    fprintf(stderr, "plen: %d; proto: %d\n", ntohs(vcmp->vcmp_vip.vip_len),
            ntohs(vcmp->vcmp_vip.vip_prot));
    fprintf(stderr, "saddr: %s.%d; ",
            inet_ntoa(vcmp->vcmp_vip.vip_src.vad_ipv4),
            ntohs(vcmp->vcmp_vip.vip_src.vad_port));
    fprintf(stderr, "daddr: %s.%d\n\n",
            inet_ntoa(vcmp->vcmp_vip.vip_dst.vad_ipv4),
            ntohs(vcmp->vcmp_vip.vip_dst.vad_port));

  } else {
    fprintf(stdout, "vcmp_input: Bad VCMP version number %d\n", ntohs(vcmp->vcmp_ver));
  }
    
  return;
}

/*
 * vcmp_output(vrt, dst, errcode, vmbp)
 *
 * dst points to the vad of the destination.
 * errcode is the vcmp errcode.
 * vmbp points to the vip header of error causing pkt.
 *
 * Assemble a vcmp pkt from the args and send it to dst.
 * Nothing is modified.
 * Return whatever vip_output returns.
*/
int
_vcmp_output(vrouter_t *vrt, vin_addr_t *dst, u_short errcode, vmbuf_t *vmbp)
{
  vcmp_t vcmp;
  vmbuf_t vmbuf;
  int err = 0;

  /* Make a vcmp packet. */
  vcmp.vcmp_ver = htons(VCMP_VERSION);
  vcmp.vcmp_code = htons(errcode);
  
  vmbuf.vmbuf_data = (char *) &vcmp;
  vmbuf.vmbuf_len = sizeof(vcmp_t)-sizeof(vip_t);
  vmbuf.vmbuf_next = vmbp;
  
  /* send it out as a vip pkt */
  err = vip_output(vrt, dst, 0, VIP_PROTOVCMP, &vmbuf);
  return(err);
}

