/*
 * Copyright (c) 1998, 2000, 2004 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Sugih Jamin (jamin@eecs.umich.edu)
*/
#ifndef lint
static char rcsid[] =
"@(#) $Id: vcmp.c,v 1.13 2000/10/13 17:14:53 sugih Exp $ (UM)";
#endif

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "vnet.h"
#include "vip.h"
#include "vcmp.h"

extern void _vcmp_input(vin_addr_t *src, vcmp_t *vcmp);
extern int _vcmp_output(vrouter_t *vrt, vin_addr_t *dst, u_short errcode, vmbuf_t *vmbp);

/* VCMP error messages, add errcode to vcmp.h when adding new messages. */
char *_vcmp_errmsg[] = {
  "Hop count exceeded",
  "Host unreachable",
  "Link down",
  "Unknown protocol",
  "Bad error code",
};

/*
 * vcmp_input(src, vcmp)
 *
 * src points to whence the vcmp pkt came from.
 * vcmp points to the vcmp pkt.
 * Both in network byte-order.
 *
 * Prints out the contents of the vcmp pkt,
 * mapping the vcmp error code to its corresponding
 * error message.  Assumes existence of
 * _vcmp_errmsg global variable.
 * Nothing is modified.
*/
void
vcmp_input(vin_addr_t *src, vcmp_t *vcmp)
{
  _vcmp_input(src, vcmp);
  return;
}

/*
 * vcmp_output(vrt, dst, errcode, vmbp)
 *
 * dst points to the vad of the destination.
 * errcode is the vcmp errcode.
 * vmbp points to the vip header of error causing pkt.
 *
 * Assemble a vcmp pkt from the args and send it to dst.
 * Nothing is modified.
 * Return whatever vip_output returns.
*/
int
vcmp_output(vrouter_t *vrt, vin_addr_t *dst, u_short errcode, vmbuf_t *vmbp)
{
  /* Make a vcmp packet. */
  /* send it out as a vip pkt */

  return(_vcmp_output(vrt, dst, errcode, vmbp));
}

