/*
 * Copyright (c) 1998, 2000 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Sugih Jamin (jamin@eecs.umich.edu)
 *
 * @(#) $Id: vcmp.h,v 1.11 2000/10/13 17:14:53 sugih Exp $ (UM)
 *
*/
#ifndef __vcmp_h__
#define __vcmp_h__

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "vnet.h"
#include "vif.h"
#include "vip.h"

/* VCMP: virtual control message protocol */
#define VCMP_VERSION 1

/* the meaning of these VCMP error types, see
   the error messages in vcmp.c */
#define VCMP_EHOPS 0
#define VCMP_EHOST 1
#define VCMP_ELINK 2
#define VCMP_EPROT 3
#define VCMP_ELAST 4

typedef struct _vcmp_hdr vcmp_t;  /* VCMP header type */

struct _vcmp_hdr {
  u_short vcmp_ver;               /* version number */
  u_short vcmp_code;              /* error code */
  vip_t vcmp_vip;                 /* vip header of error causing pkt */
};

extern char *_vcmp_errmsg[];

/*
 * vcmp_errmsg(int errcode)
 *
 * Returns the error message for errcode.
 * Errcode must be a valid index off the global
 * _vcmp_errmsg array of error messages.
 * Nothing is modified.
*/
#define vcmp_errmsg(errcode) (_vcmp_errmsg[errcode])

extern void vcmp_input(vin_addr_t *src, vcmp_t *vcmp);
extern int vcmp_output(vrouter_t *vrt, vin_addr_t *dst, u_short errcode, vmbuf_t *vmbp);

#endif /*__vcmp_h__*/
