/*
 * Copyright (c) 1997, 1998, 2000 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Sugih Jamin (jamin@eecs.umich.edu)
 *
 * @(#) $Id: vif.h,v 1.15 2000/10/24 21:47:58 sugih Exp $ (UM)
 *
*/
#ifndef __vif_h__
#define __vif_h__

#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>

#include "vnet.h"

/* VIF: virtual interface (protocol) */
#define VIF_PROTOVRC 0xff
#define VIF_PROTOVIP 0xfe

#define VIF_VERSION 1
#define VRC_VERSION 1
#define VIF_ERRVERS -1  /* pkt contains unknown version number */

/* the following are indices into the virtual interface (vif) table (vit) */
#define VIF_CONIF 0     /* index to the configuration descriptors */
#define VIF_MINIF 1     /* index to the first vif connected to other vrouter */
#define VIF_MAXIF 8     /* index to the last vif connected to other vrouter */

/* the state of a vif */
#define VIF_OFFLINE 0     /* uninitialized vif */
#define VIF_INCOMING 0x1  /* incoming line connected */
#define VIF_OUTGOING 0x2  /* outgoing line connected */
#define VIF_LINKUP 0x3    /* vif coming up */
#define VIF_ONLINE 0x7    /* duplex connected vif */
#define VIF_LINKDOWN 0x8  /* vif being torn down */
#define VIF_LINKRC 0x10   /* vif being reconfigured */

#define VIF_DIRECT 1      /* direct connection */

#define VIF_TIMEOUT 0xff  /* timeout waiting for event */

typedef struct _vif_hdr vih_t;   /* vif pkt header */
typedef struct _vif_rc vrc_t;    /* vif reconfiguration pkt */
typedef struct _vif vif_t;       /* a vif */
typedef struct _vif_tbl vit_t;   /* a vif table */

struct _vif_rc {
  u_short vrc_vers;        /* protocol version */
  u_short vrc_nvad;        /* number of virtual addresses (vads), see vin.h */
  vin_addr_t vrc_vad;      /* first vad, vad_metric indicates link up or down */
};

struct _vif_hdr {
  u_short vih_vers;        /* protocol version */
  u_short vih_prot;        /* next layer protocol */
};

struct _vif {
  int vif_ind;             /* the socket descriptor of the input interface */
  int vif_outd;            /* the socket descriptor of the output interface */
  int vif_status;          /* incoming, outgoing, up, down, online, offline */
  vin_addr_t vif_vad;      /* the vad of this vif */
};

struct _vif_tbl {
  int vit_maxifd;              /* max descriptor number */
  int vit_nvif;                /* number of incoming virtual interfaces */
  vif_t vit_ifs[VIF_MAXIF+1];  /* virtual interfaces */
};

/*
 * int
 * vif_nvif(vit_t *vit)
 *
 * Returns the number of interfaces in vit.
 * Nothing is modified.
*/
#define vit_nvif(vit) ((vit) ? (vit)->vit_nvif : 0)

/*
 * int
 * vif_isonline(vif_t *vif)
 *
 * Returns 1 if vif is VIF_ONLINE, 0 otherwise.
 * Nothing is modified.
*/
#define vif_isonline(vif) ((vif) && (vif)->vif_status == VIF_ONLINE)

/*
 * int
 * vif_isdown(vif_t *vif)
 *
 * Returns 1 if vif is VIF_LINKDOWN, 0 otherwise.
 * Nothing is modified.
*/
#define vif_isdown(vif) ((vif) && (vif)->vif_status == VIF_LINKDOWN)

/*
 * int
 * vif_isoffline(vif_t *vif)
 *
 * Returns 1 if vif is VIF_OFFLINE, 0 otherwise.
 * Nothing is modified.
*/
#define vif_isoffline(vif) ((vif) && (vif)->vif_status == VIF_OFFLINE)

extern void vif_init(int rcif, int port, vit_t *vit);
extern vif_t *vif_input(vin_addr_t *self, vit_t *vit, struct timeval *timer);
extern int vif_pullup(vif_t *vif, char *buf, int len);
extern int vif_pullupn(vif_t *vif, char *buf, int len);
extern int vif_output(vif_t *vif, u_short prot, vmbuf_t *vmbp);
extern void vif_offline(vif_t *vif);
extern int vif_online(vit_t *vit, vif_t **up);

#endif /*__vif_h__*/
