/*
 * Copyright (c) 1997, 1998 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Sugih Jamin (jamin@eecs.umich.edu)
 *
 * @(#) $Id: vin.h,v 1.3 1998/01/24 12:55:10 sugih Exp $ (UM)
 *
*/
#ifndef __vin_h__
#define __vin_h__

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

typedef struct _vin_addr vin_addr_t;

struct _vin_addr {
  struct in_addr vad_ipv4;   /* ipv4 address */
  u_short vad_port;          /* port number, always kept in network byte-order */
  u_short vad_metric;        /* to be interpreted by protocol using the field */
};

/*
 * int
 * vin_eq(vin_addr_t *vada, vin_addr_t *vadb)
 * 
 * Returns 1 if vada contains the same data as vadb,
 * 0 otherwise.
*/
#define vin_eq(vada, vadb) ((vada)->vad_ipv4.s_addr == (vadb)->vad_ipv4.s_addr && \
                            (vada)->vad_port == (vadb)->vad_port)

/*
 * int
 * vin_lt(vin_addr_t *vada, vin_addr_t *vadb)
 * 
 * Returns 1 if the ipv4 address of vada is
 * numerically smaller than the ipv4 address of
 * vadb, 0 otherwise.
 * Nothing is modified.
*/
#define vin_lt(vada, vadb) (((u_long) (vada)->vad_ipv4.s_addr) < ((u_long) (vadb)->vad_ipv4.s_addr))

#endif /*__vin_h__*/
