/*
 * Copyright (c) 1997, 1998, 2000 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Sugih Jamin (jamin@eecs.umich.edu)
 *
 * @(#) $Id: vip.h,v 1.15 2000/10/13 17:14:53 sugih Exp $ (UM)
 *
*/
#ifndef __vip_h__
#define __vip_h__

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "vnet.h"
#include "vif.h"
#include "vrp.h"

/* VIP: virtual internetwork protocol */
#define VIP_VERSION 8        
#define VIP_MAXPKTSIZE 65535  /* maximum packet size, excluding header */

/* VCMP: virtual control message protocol */
#define VIP_PROTOVCMP 1       /* protocol number: 1 */

/* VRP: virtual routing protocol */
#define VIP_PROTOVRP 2        /* protocol number: 2 */

/* VTP: virtual transport protocol */
#define VIP_PROTOVTP 4        /* protocol number: 4 */

typedef struct _vip_hdr vip_t; /* vip pkt header */

struct _vip_hdr {
  u_short vip_ver;      /* version number */
  u_short vip_hop;      /* hop limit */
  u_short vip_len;      /* packet size, excluding header */
  u_short vip_prot;     /* protocol number */
  vin_addr_t vip_src;   /* source vin address, see vnet.h */
  vin_addr_t vip_dst;   /* destination vin address */
};

extern void vip_input(vif_t *vif, vrouter_t *vrt);
extern void vip_forw(vrouter_t *vrt, vip_t *vp, vmbuf_t *vmbp);
extern int vip_output(vrouter_t *vrt, vin_addr_t *dst, u_short ttl, u_short proto, vmbuf_t *vmbp);
extern void vip_drop(vrouter_t *vrt, vip_t *vp, char *data, u_short errcode);

#endif /*__vip_h__*/
