/*
 * Copyright (c) 1998,2000 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Sugih Jamin (jamin@eecs.umich.edu)
 *
 * @(#) $Id: vnet.h,v 1.6 2000/10/13 17:14:53 sugih Exp $ (UM)
 *
*/
#ifndef __vnet_h__
#define __vnet_h__

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

/*#define VNET_PA1*/
#define VNET_MINPORT 5001      /* minimum port */
#define VNET_MAXPORT 32767     /* maximum port */
#define VROUTER_PORT 9841      /* default vrouter port */
#define SDR_PORT 8419        /* default seeder port */
#define VNET_BLEN 128        /* default buffer size for messages */

typedef struct _vin_addr vin_addr_t;
typedef struct _vmbuf vmbuf_t;

struct _vmbuf {
  vmbuf_t *vmbuf_next;         /* upper layer(s)' data */
  int vmbuf_len;               /* size of vmbuf_data in bytes */
  char *vmbuf_data;            /* data from this layer */
};

struct _vin_addr {
  struct in_addr vad_ipv4;   /* ipv4 address */
  u_short vad_port;          /* port number, always kept in network byte-order */
  u_short vad_metric;        /* to be interpreted by protocol using the field */
};

/*
 * int
 * vin_eq(vin_addr_t *vada, vin_addr_t *vadb)
 * 
 * Returns 1 if vada contains the same data as vadb,
 * 0 otherwise.  Nothing is modified.
*/
#define vin_eq(vada, vadb) ((vada)->vad_ipv4.s_addr == (vadb)->vad_ipv4.s_addr && \
                            (vada)->vad_port == (vadb)->vad_port)

/*
 * void
 * vnet_assert(int err, char *errmsg)
 *
 * If err is 1, outputs errmsg, dumps
 * core and kills process, else returns
 * without modifying anything.
 *
 * You can use the core file to figure out what the
 * problem is.   For example, you can use gdb to
 * inspect the body and conduct post-mortem investigation,
 * so to speak:
 *  % gdb vrouted core
*/
#define vnet_assert(err, errmsg) { if (err) { perror(errmsg);/* exit(errno) */abort(); } }

/*
 * int
 * time_diff(struct timeval ots, struct timeval nts)
 *
 * Assume nts > ots.
 * Returns the ceiling on the number of seconds passed between ots and nts.
 * Minimum value returned is 1.
*/
#define USECSPERMSEC 1000
#define USECSPERSEC 1000000
#define time_diff(ots, nts) ((nts).tv_sec - (ots).tv_sec + (USECSPERSEC - (ots).tv_usec + (nts).tv_usec)/USECSPERSEC)
/*
#define time_add_ms(ts1, ts2, ms) \
      { int _usec_delta = ((ts2).tv_usec + (ms)*USECPERMSEC);\
        if (_time_delta >= USECPERSEC ) { \
          (ts1).tv_sec = (ts2).tv_sec+1; \
          (ts1).tv_usec = _usec_delta - USECPERSEC; \
         } else {\
          (ts1).tv_sec = (ts2).tv_sec;\
          (ts1).tv_usec = _usec_delta;\ 
         }\
      }
*/

#endif /*__vnet_h__*/
