/*
 * Copyright (c) 1997,1998, 2000, 2004 University of Michigan, Ann Arbor.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of Michigan, Ann Arbor. The name of the University 
 * may not be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Sugih Jamin (jamin@eecs.umich.edu)
*/
#ifndef lint
static char rcsid[] =
"@(#) $Id: vtp.c,v 1.9 2001/09/17 15:22:51 sugih Exp $ (UM)";
#endif

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "vnet.h"
#include "vip.h"
#include "vtp.h"

/*
 * vtp_input(src, data, len):
 * Takes a pointer to source address (in network byte-order)
 * and a pointer to character buffer of length len.
 * Writes buffer to stdout.
*/
void
vtp_input(vin_addr_t *src, char *data, int len)
{
  struct hostent *host;

  host = gethostbyaddr((char *) &src->vad_ipv4,
                      sizeof(struct in_addr), AF_INET);
  fprintf(stdout, "VTP pkt received from %s.%d:\n", host->h_name, ntohs(src->vad_port));
  fwrite(data, sizeof(char), len, stdout);
  fprintf(stdout, "\n");
  return;
}

/*
 * vtp_drop(src, data, len):
 * Takes a pointer to source address (in network byte-order)
 * and a pointer to character buffer of length len.
 * Writes buffer to stdout.
*/
void
vtp_drop(vin_addr_t *src, char *data, int len)
{
  struct hostent *host;

  host = gethostbyaddr((char *) &src->vad_ipv4,
                      sizeof(struct in_addr), AF_INET);
  fprintf(stdout, "VTP pkt from %s.%d is dropped, pkt content:\n", host->h_name, ntohs(src->vad_port));
  fwrite(data, sizeof(char), len, stdout);
  fprintf(stdout, "\n");
  return;
}

/*
 * vtp_output: creates a vmbuf_t for data and
 * sends it out through vip_output.
 *
 * Doesn't modify anything.
*/
void
vtp_output(vrouter_t *vrt, vin_addr_t *dst, u_short ttl, char *data, int len)
{
  int err;
  vmbuf_t vmbuf;

  vmbuf.vmbuf_data = data;
  vmbuf.vmbuf_len = len;
  vmbuf.vmbuf_next = 0;
  
  if (vip_output(vrt, dst, ttl, VIP_PROTOVTP, &vmbuf)) {
    fprintf(stderr, "vtp_output: vip_output returns error.\n");
  }

  return;  
}

/*
 * vtp_test: an "application-level" function
 * that generates data and sends it out as a vtp
 * packet.
 * Do nothing if there is only one vrouter.
 * If there are two vrouters, doesn't test vcmp.
 * If there are more than 2 vrouters, test vcmp for the
 * latter half of VTP_TESTNUM.
*/
#define VTP_TESTNUM 2
void
vtp_test(vrouter_t *vrt)
{
  static int seq;
  int i, j;
  vin_addr_t *self;
  vif_t *vif;
  char buf[VNET_BLEN];
  struct hostent *host;

  self = vrp_self(vrt);
  /* i = vrp_rand_dest(vrt); */

  /* find the furthest destination. */
  for (i = 1, j = 0; i < vrt->vrouter_nent; i++) {
    if (vif_isonline(vrt->vrouter_rt[i].vrtbl_vif) &&
        (!j || (vrt->vrouter_rt[j].vrtbl_cost < vrt->vrouter_rt[i].vrtbl_cost))) {
      j = i;
    }
  }

  if (j && seq < VTP_TESTNUM) {
    vif = vrt->vrouter_rt[j].vrtbl_vif;

    if (vif) {
      host = gethostbyaddr((char *) &self->vad_ipv4, sizeof(struct in_addr), AF_INET);
      sprintf(buf, "Seq#: %d\nSrc: %s.%d,", seq, host->h_name, ntohs(self->vad_port));
      host = gethostbyaddr((char *) &vrt->vrouter_rt[j].vrtbl_dst.vad_ipv4,
                           sizeof(struct in_addr), AF_INET);
      sprintf(&buf[strlen(buf)], " Dest: %s.%d\n", host->h_name,
              ntohs(vrt->vrouter_rt[j].vrtbl_dst.vad_port));
      
      printf("vtp_test: pkt %d sent with hop count %d.\n", seq, (seq < VTP_TESTNUM/2 ? 0 : 1));
      printf("%s\n", buf);
      vtp_output(vrt, &vrt->vrouter_rt[j].vrtbl_dst, (seq < VTP_TESTNUM/2 ? 0 : 1), buf, strlen(buf)+1);
    }
    seq++;
  }

  return;
}
